require_relative 'spec_helper'

describe RbEvent do
	let(:klass) do
		Class.new do
			include RbEvent

			def fire(name, *args)
				fire_event(name, *args)
			end

			attr_accessor :flag
			def set_flag(val)
				@flag = val
			end
		end
	end

	let(:instance) do
		klass.new
	end

	describe 'RbEvent#listen' do
		it 'pass block' do
			flag = false
			instance.listen :listen_test do |val|
				flag = val
			end
			instance.fire :listen_test, true
			flag.should be true
		end

		it 'pass proc' do
			flag = false
			instance.listen(:listen_test, proc { |val| flag = val })
			instance.fire :listen_test, true
			flag.should be true
		end

		it 'pass method' do
			instance.flag = false
			instance.listen :listen_test, instance.method(:set_flag)
			instance.fire :listen_test, true
			instance.flag.should be true
		end
	end

	describe 'RbEvent#unlisten' do
		it 'pass listener' do
			flag = false
			l = instance.listen :unlisten_test do |val|
				flag = val
			end
			instance.unlisten :unlisten_test, l
			instance.fire :unlisten_test, true
			flag.should be false
		end

		it 'pass method' do
			instance.flag = false
			instance.listen :unlisten_test, instance.method(:set_flag)
			instance.unlisten :unlisten_test, instance.method(:set_flag)
			instance.fire :unlisten_test, true
			instance.flag.should be false
		end
	end

	describe 'RbEvent#listen?' do
		it 'pass listener' do
			l = instance.listen(:has_listener_test) {}
			instance.listen?(:has_listener_test, l).should be true
		end

		it 'pass method' do
			l = instance.listen :has_listener_test, instance.method(:set_flag)
			instance.listen?(:has_listener_test, instance.method(:set_flag)).should be true
		end
	end

	describe 'RbEvent#fire_event' do
		it 'only call events of the name' do
			flag = false
			instance.listen :an_event do
				flag = true
			end
			instance.fire :another_event
			flag.should be false
		end
	end
end
